=begin
 sculpteo - Sculpteo tools for Sketchup 
 adds tool to upload to the Sculpteo website

 Copyright (C) 2010 Sculpteo
 uuid generation from uuid.rb Copyright(c) 2005 URABE, Shyouhei.

=end

require "sketchup.rb"

module Sculpteo

  #-- Constants --
#  SCULPTEO_SERVER = "http://192.168.1.7:8000"
  SCULPTEO_SERVER = "http://www.sculpteo.com"
  SCULPTEO_UPLOADFILE = "gallery/uploadfile"
  SCULPTEO_UPLOAD = "gallery/3D/upload"

  # UUID  generation  using  random-number  generator.   From  it's  random
  # nature, there's  no warranty that  the created ID is  really universaly
  # unique.
  def self.mask ver, str
    ver = ver & 15
    v = str[6]
    v &= 0b0000_1111
    v |= ver << 4
    str[6] = v
    r = str[8]
    r &= 0b0011_1111
    r |= 0b1000_0000
    str[8] = r
    str
  end
	
  def self.prand
    rand 0x100000000
  end
  
  def self.uuid
    rnd = [prand(), prand(), prand(), prand()].pack "N4"
    raw = mask(4, rnd)
    tmp = raw.unpack "C*"
    return "%02x%02x%02x%02x%02x%02x%02x%02x" % tmp
  end

  def self.multipart_encode key, filename
    content = open(filename, "rb") do |f|
      f.read()
    end

    return "Content-Disposition: form-data; name=" + key + "; filename=" + filename + "rn" +
          #"Content-Transfer-Encoding: binaryrn" +
          "Content-Type: application/octet-streamrnrn" +
          content + "rn"
  end

  def self.upload

    # no native Ruby, must handle tempdir creation...
    tempdir = ENV["TEMP"] + "/"
    tempdir = ENV["TMPDIR"] if not tempdir
    tempdir = tempdir.tr("\\", "/")

    uuid = uuid()
    tempname = "sculpteo" + uuid + ".skp"
    tempfilename = tempdir + tempname

    # save the active model
    Sketchup.set_status_text "Exporting..."

    puts "Exporting to " + tempfilename
    model = Sketchup.active_model
    modelname = model.name
    status = model.save tempfilename

    # determine language
    lang = "en"
    locale = Sketchup.get_locale
    if (locale)
      if (locale[0,2].casecmp("en") == 0)
        lang = "en"
      end
      if (locale[0,2].casecmp("fr") == 0)
        lang = "fr"
      end
    end

    # post the design
    Sketchup.set_status_text "Uploading..."

    post_url = SCULPTEO_SERVER + "/" + lang + "/" + SCULPTEO_UPLOADFILE + "/?X-Progress-ID=" + uuid

    content = File.open(tempfilename, "rb") {|io| io.read }
    # urlsafe base64 encoding
    content64 = content.unpack("a*").pack("m").tr('+/', '-_')
    post_data = "filename=" + tempname + "&file=" + content64

    post_dlg = UI::WebDialog.new("Uploading...", true,
                                 "", 100, 100, 50, 50, true);
    post_dlg.show
    post_dlg.post_url(post_url, post_data)
    #post_dlg.close

    if (modelname == '')
      modelname = 'Sketchup'
    end

    # show the upload page
    url = SCULPTEO_SERVER + "/" + lang + "/" + SCULPTEO_UPLOAD + "/?X-Progress-ID=" + uuid + "&name=" + modelname
    UI.openURL(url)

    # erase temporary model
    File.unlink(tempfilename)

    Sketchup.set_status_text "Ready"

  end

end # module Sculpteo

UI.menu("Plugins").add_submenu("Sculpteo").add_item("Upload") { Sculpteo::upload }

toolbar = UI::Toolbar.new "Sculpteo"
cmd = UI::Command.new("Upload") { 
  Sculpteo::upload
}
cmd.small_icon = "sculpteo_upload_small.png"
cmd.large_icon = "sculpteo_upload_large.png"
cmd.tooltip = "Upload to Sculpteo"
cmd.status_bar_text = "Uploading..."
cmd.menu_text = "Upload to Sculpteo"
toolbar = toolbar.add_item cmd
toolbar.show
